  
#ifndef _atmosphere_cmpt_h_
#define _atmosphere_cmpt_h_

#include <vector>
#include <string>
#include <complex>
#include <stdcasa/record.h>
#include <tools/casaswig_types.h>
#include <casa/Exceptions/Error.h>
#include <tools/atmosphere/atmosphere_forward.h>

using namespace std;

namespace casac {
           


   class  atmosphere  {
       public :
static Quantity _altitude_getBasicAtmParms;
static Quantity _temperature_getBasicAtmParms;
static Quantity _pressure_getBasicAtmParms;
static Quantity _maxAltitude_getBasicAtmParms;

static double _humidity_getBasicAtmParms;
		static Quantity _dTem_dh_getBasicAtmParms;
static Quantity _dP_getBasicAtmParms;

static double _dPm_getBasicAtmParms;
		static Quantity _h0_getBasicAtmParms;

static string _atmType_getBasicAtmParms;
		static Quantity _thickness_getProfile;
static Quantity _temperature_getProfile;
static Quantity _watermassdensity_getProfile;
static Quantity _water_getProfile;
static Quantity _pressure_getProfile;
static Quantity _O3_getProfile;
static Quantity _CO_getProfile;
static Quantity _N2O_getProfile;
static std::vector<double> _dryOpacity_getDryOpacitySpec;
static Quantity _wetOpacity_getWetOpacitySpec;
static Quantity _tebbSky_getTebbSkySpec;
static Quantity _trjSky_getTrjSkySpec;
atmosphere();
         ~atmosphere();
         string getAtmVersion();
         std::vector<std::string> listAtmosphereTypes();
         string initAtmProfile( const Quantity& altitude=Quantity(std::vector<double>(1, 5000.),"m"),  const Quantity& temperature=Quantity(std::vector<double>(1, 270.0),"K"),  const Quantity& pressure=Quantity(std::vector<double>(1, 560.0),"mbar"),  const Quantity& maxAltitude=Quantity(std::vector<double>(1, 48.0),"km"), 
			  double humidity=20.0,  const Quantity& dTem_dh=Quantity(std::vector<double>(1, -5.6),"K/km"),  const Quantity& dP=Quantity(std::vector<double>(1, 10.0),"mbar"), 
			  double dPm=1.2,  const Quantity& h0=Quantity(std::vector<double>(1, 2.0),"km"),  int  atmType=1);
         string updateAtmProfile( const Quantity& altitude=Quantity(std::vector<double>(1, 5000.),"m"),  const Quantity& temperature=Quantity(std::vector<double>(1, 270.0),"K"),  const Quantity& pressure=Quantity(std::vector<double>(1, 560.0),"mbar"), 
			  double humidity=20.0,  const Quantity& dTem_dh=Quantity(std::vector<double>(1, -5.6),"K/km"),  const Quantity& h0=Quantity(std::vector<double>(1, 2.0),"km"));
         string getBasicAtmParms( Quantity& _altitude=atmosphere::_altitude_getBasicAtmParms,  Quantity& _temperature=atmosphere::_temperature_getBasicAtmParms,  Quantity& _pressure=atmosphere::_pressure_getBasicAtmParms,  Quantity& _maxAltitude=atmosphere::_maxAltitude_getBasicAtmParms, 
			  double& _humidity=atmosphere::_humidity_getBasicAtmParms,  Quantity& _dTem_dh=atmosphere::_dTem_dh_getBasicAtmParms,  Quantity& _dP=atmosphere::_dP_getBasicAtmParms, 
			  double& _dPm=atmosphere::_dPm_getBasicAtmParms,  Quantity& _h0=atmosphere::_h0_getBasicAtmParms, string& _atmType=atmosphere::_atmType_getBasicAtmParms);
         int getNumLayers();
         Quantity getGroundWH2O();
         string getProfile( Quantity& _thickness=atmosphere::_thickness_getProfile,  Quantity& _temperature=atmosphere::_temperature_getProfile,  Quantity& _watermassdensity=atmosphere::_watermassdensity_getProfile,  Quantity& _water=atmosphere::_water_getProfile,  Quantity& _pressure=atmosphere::_pressure_getProfile,  Quantity& _O3=atmosphere::_O3_getProfile,  Quantity& _CO=atmosphere::_CO_getProfile,  Quantity& _N2O=atmosphere::_N2O_getProfile);
         int initSpectralWindow( int  nbands=1,  const Quantity& fCenter=Quantity(std::vector<double>(1, 90),"GHz"),  const Quantity& fWidth=Quantity(std::vector<double>(1, 0.64),"GHz"),  const Quantity& fRes=Quantity(std::vector<double>(1, 0.0),"GHz"));
         int addSpectralWindow( const Quantity& fCenter=Quantity(std::vector<double>(1, 350),"GHz"),  const Quantity& fWidth=Quantity(std::vector<double>(1, 0.008),"GHz"),  const Quantity& fRes=Quantity(std::vector<double>(1, 0.002),"GHz"));
         int getNumSpectralWindows();
         int getNumChan( int  spwid=0);
         int getRefChan( int  spwid=0);
         Quantity getRefFreq( int  spwid=0);
         Quantity getChanSep( int  spwid=0);
         Quantity getChanFreq( int  chanNum=0,  int  spwid=0);
         Quantity getSpectralWindow( int  spwid=0);
         double getChanNum( const Quantity& freq,  int  spwid=0);
         Quantity getBandwidth( int  spwid=0);
         Quantity getMinFreq( int  spwid=0);
         Quantity getMaxFreq( int  spwid=0);
         double getDryOpacity( int  nc=-1,  int  spwid=0);
         double getDryContOpacity( int  nc=-1,  int  spwid=0);
         double getO2LinesOpacity( int  nc=-1,  int  spwid=0);
         double getO3LinesOpacity( int  nc=-1,  int  spwid=0);
         double getCOLinesOpacity( int  nc=-1,  int  spwid=0);
         double getN2OLinesOpacity( int  nc=-1,  int  spwid=0);
         Quantity getWetOpacity( int  nc=-1,  int  spwid=0);
         double getH2OLinesOpacity( int  nc=-1,  int  spwid=0);
         double getH2OContOpacity( int  nc=-1,  int  spwid=0);
         int getDryOpacitySpec( int  spwid=0,  std::vector<double>& _dryOpacity=atmosphere::_dryOpacity_getDryOpacitySpec);
         int getWetOpacitySpec( int  spwid=0,  Quantity& _wetOpacity=atmosphere::_wetOpacity_getWetOpacitySpec);
         Quantity getDispersivePhaseDelay( int  nc=-1,  int  spwid=0);
         Quantity getDispersiveWetPhaseDelay( int  nc=-1,  int  spwid=0);
         Quantity getNonDispersiveWetPhaseDelay( int  nc=-1,  int  spwid=0);
         Quantity getNonDispersiveDryPhaseDelay( int  nc=-1,  int  spwid=0);
         Quantity getNonDispersivePhaseDelay( int  nc=-1,  int  spwid=0);
         Quantity getDispersivePathLength( int  nc=-1,  int  spwid=0);
         Quantity getDispersiveWetPathLength( int  nc=-1,  int  spwid=0);
         Quantity getNonDispersiveWetPathLength( int  nc=-1,  int  spwid=0);
         Quantity getNonDispersiveDryPathLength( int  nc=-1,  int  spwid=0);
         Quantity getO2LinesPathLength( int  nc=-1,  int  spwid=0);
         Quantity getO3LinesPathLength( int  nc=-1,  int  spwid=0);
         Quantity getCOLinesPathLength( int  nc=-1,  int  spwid=0);
         Quantity getN2OLinesPathLength( int  nc=-1,  int  spwid=0);
         Quantity getNonDispersivePathLength( int  nc=-1,  int  spwid=0);
         Quantity getAbsH2OLines( int  nl,  int  nf=0,  int  spwid=0);
         Quantity getAbsH2OCont( int  nl,  int  nf=0,  int  spwid=0);
         Quantity getAbsO2Lines( int  nl,  int  nf=0,  int  spwid=0);
         Quantity getAbsDryCont( int  nl,  int  nf=0,  int  spwid=0);
         Quantity getAbsO3Lines( int  nl,  int  nf=0,  int  spwid=0);
         Quantity getAbsCOLines( int  nl,  int  nf=0,  int  spwid=0);
         Quantity getAbsN2OLines( int  nl,  int  nf=0,  int  spwid=0);
         Quantity getAbsTotalDry( int  nl,  int  nf=0,  int  spwid=0);
         Quantity getAbsTotalWet( int  nl,  int  nf=0,  int  spwid=0);
         bool setUserWH2O( const Quantity& wh2o=Quantity(std::vector<double>(1, 0.0),"mm"));
         Quantity getUserWH2O();
         bool setAirMass(
			  double airmass);
         double getAirMass();
         bool setSkyBackgroundTemperature( const Quantity& tbgr=Quantity(std::vector<double>(1, 2.73),"K"));
         Quantity getSkyBackgroundTemperature();
         Quantity getAverageTebbSky( int  spwid=0,  const Quantity& wh2o=Quantity(std::vector<double>(1, -1),"mm"));
         Quantity getTebbSky( int  nc=-1,  int  spwid=0,  const Quantity& wh2o=Quantity(std::vector<double>(1, -1),"mm"));
         int getTebbSkySpec( int  spwid=0,  const Quantity& wh2o=Quantity(std::vector<double>(1, -1),"mm"),  Quantity& _tebbSky=atmosphere::_tebbSky_getTebbSkySpec);
         Quantity getAverageTrjSky( int  spwid=0,  const Quantity& wh2o=Quantity(std::vector<double>(1, -1),"mm"));
         Quantity getTrjSky( int  nc=-1,  int  spwid=0,  const Quantity& wh2o=Quantity(std::vector<double>(1, -1),"mm"));
         int getTrjSkySpec( int  spwid=0,  const Quantity& wh2o=Quantity(std::vector<double>(1, -1),"mm"),  Quantity& _trjSky=atmosphere::_trjSky_getTrjSkySpec);

      private :
#include <tools/atmosphere/atmosphere_private.h>
   }; 

 
}
#endif
