  
#ifndef _componentlist_cmpt_h_
#define _componentlist_cmpt_h_

#include <vector>
#include <string>
#include <complex>
#include <stdcasa/record.h>
#include <tools/casaswig_types.h>
#include <casa/Exceptions/Error.h>
#include <tools/components/componentlist_forward.h>

using namespace std;

namespace casac {
           


   class  componentlist  {
       public :
componentlist();
         ~componentlist();
         bool open(const  string& filename=string(""),  bool  nomodify=false,  bool  log=true);
         int asciitocomponentlist(const  string& filename, const  string& asciifile, const  string& refer=string("J2000"), const  string& format=string("ST"),  const record& direction=initialize_record("") ,  const record& spectrum=initialize_record("") ,  const record& flux=initialize_record("") ,  bool  log=true);
         bool concatenate( const variant& list= initialize_variant(""),  const std::vector<int>& which=initialize_vector(1, (int)-1),  bool  log=true);
         bool fromrecord( const record& record=initialize_record("") );
         record* torecord();
         bool remove( const std::vector<int>& which=initialize_vector(1, (int)-1),  bool  log=true);
         bool purge();
         bool recover( bool  log=true);
         int length();
         std::vector<int> indices();
         bool sort(const  string& criteria=string("Flux"),  bool  log=true);
         bool isphysical( const std::vector<int>& which=initialize_vector(1, (int)-1));
         std::vector<double>sample( const variant& direction= initialize_variant("J2000 00h00m00.00 90d00m00.0"),  const variant& pixellatsize= initialize_variant("0.0deg"),  const variant& pixellongsize= initialize_variant("0.0deg"),  const variant& frequency= initialize_variant("1.4GHz"));
         bool rename(const  string& filename,  bool  log=true);
         bool simulate( int  howmany=1,  bool  log=true);
         bool addcomponent( const variant& flux= initialize_variant(""), const  string& fluxunit=string("Jy"), const  string& polarization=string("Stokes"),  const variant& dir= initialize_variant("J2000 00h00m00.0 90d00m00.0"), const  string& shape=string("point"),  const variant& majoraxis= initialize_variant("2.0arcmin"),  const variant& minoraxis= initialize_variant("1.0arcmin"),  const variant& positionangle= initialize_variant("0.0deg"),  const variant& freq= initialize_variant("LSRK 1.415GHz"), const  string& spectrumtype=string("constant"), 
			  double index=1.0,  const std::vector<double>& optionalparms=initialize_vector(1, (double)0.0), const  string& label=string(""));
         bool close( bool  log=true);
         bool edit( int  which,  bool  log=true);
         bool done();
         bool select( const std::vector<int>& which);
         bool deselect( const std::vector<int>& which);
         std::vector<int> selected();
         string getlabel( int  which);
         bool setlabel( int  which, const  string& value,  bool  log=true);
         std::vector<double>getfluxvalue( int  which);
         string getfluxunit( int  which);
         string getfluxpol( int  which);
         std::vector<double>getfluxerror( int  which);
         bool setflux( int  which,  const variant& value= initialize_variant(""), const  string& unit=string("Jy"), const  string& polarization=string("Stokes"),  const variant& error= initialize_variant(""),  bool  log=true);
         bool convertfluxunit( int  which, const  string& unit=string("Jy"));
         bool convertfluxpol( int  which, const  string& polarization=string("Stokes"));
         record* getrefdir( int  which);
         string getrefdirra( int  which, const  string& unit=string("deg"),  int  precision=6);
         string getrefdirdec( int  which, const  string& unit=string("deg"),  int  precision=6);
         string getrefdirframe( int  which);
         bool setrefdir( int  which=1,  const variant& ra= initialize_variant(""),  const variant& dec= initialize_variant(""),  bool  log=true);
         bool setrefdirframe( int  which, const  string& frame,  bool  log=true);
         bool convertrefdir( int  which, const  string& frame);
         string shapetype( int  which);
         record* getshape( int  which);
         bool setshape( int  which, const  string& type=string("Point"),  const variant& majoraxis= initialize_variant("1.0arcmin"),  const variant& minoraxis= initialize_variant("1.0arcmin"),  const variant& positionangle= initialize_variant("0.0deg"),  const variant& majoraxiserror= initialize_variant("0.0arcmin"),  const variant& minoraxiserror= initialize_variant("0.0arcmin"),  const variant& positionangleerror= initialize_variant("0.0deg"),  const std::vector<double>& optionalparms=initialize_vector(1, (double)0.0),  bool  log=true);
         bool convertshape( int  which, const  string& majoraxis=string("arcmin"), const  string& minoraxis=string("arcmin"), const  string& positionangle=string("deg"));
         string spectrumtype( int  which);
         record* getspectrum( int  which);
         bool setstokesspectrum( int  which, const  string& type=string("spectral index"),  const std::vector<double>& index=initialize_vector(1,(double)0.0),  const std::vector<double>& tabularfreqs=initialize_vector(1,(double)1.0e11),  const std::vector<double>& tabulari=initialize_vector(1,(double)1.0),  const std::vector<double>& tabularq=initialize_vector(1,(double)0.0),  const std::vector<double>& tabularu=initialize_vector(1,(double)0.0),  const std::vector<double>& tabularv=initialize_vector(1,(double)0.0),  const variant& reffreq= initialize_variant("1.4GHz"), const  string& frame=string("LSRK"));
         bool setspectrum( int  which, const  string& type=string("spectral index"), 
			  double index=0.0,  const std::vector<double>& tabularfreqs=initialize_vector(1,(double)1.0e11),  const std::vector<double>& tabularflux=initialize_vector(1,(double)1.0), const  string& tabularframe=string("LSRK"));
         record* getfreq( int  which);
         double getfreqvalue( int  which);
         string getfrequnit( int  which);
         string getfreqframe( int  which);
         bool setfreq( int  which, 
			  double value, const  string& unit=string("'GHz'"),  bool  log=true);
         bool setfreqframe( int  which, const  string& frame=string("LSRK"),  bool  log=true);
         bool convertfrequnit( int  which, const  string& unit=string("'GHz'"));
         record* getcomponent( int  which,  bool  iknow=false);
         bool add( const record& thecomponent,  bool  iknow=true);
         bool replace( int  which,  const record& list,  const std::vector<int>& whichones=initialize_vector(1, (int)-1));
         bool summarize( int  which=-1);
         bool iscomponentlist( const variant& tool);

      private :
#include <tools/components/componentlist_private.h>
   }; 

 
}
#endif
