  
#ifndef _image_cmpt_h_
#define _image_cmpt_h_

#include <vector>
#include <string>
#include <complex>
#include <stdcasa/record.h>
#include <tools/casaswig_types.h>
#include <casa/Exceptions/Error.h>
#include <coordsys_cmpt.h>
#include <tools/images/image_forward.h>

using namespace std;

namespace casac {
           


   class  image  {
       public :
image();
         ~image();
         
                                      casac::image* newimage(const  string& infile);
         
                                      casac::image* newimagefromfile(const  string& infile);
         
                                      casac::image* imagecalc(const  string& outfile=string(""), const  string& pixels=string(""),  bool  overwrite=false, const  string& imagemd=string(""));
         
                                      casac::image* collapse(const  string& function=string(""),  const variant& axes= initialize_variant("0"), const  string& outfile=string(""),  const variant& region = initialize_variant(""), const  string& box=string(""), const  string& chans=string(""), const  string& stokes=string(""), const  string& mask=string(""),  bool  overwrite=false,  bool  stretch=false);
         
                                      casac::image* decimate(const  string& outfile=string(""),  int  axis=0,  int  factor=1, const  string& method=string("copy"),  const variant& region = initialize_variant(""), const  string& mask=string(""),  bool  overwrite=false,  bool  stretch=false);
         
                                      casac::image* imageconcat(const  string& outfile=string(""),  const variant& infiles= initialize_variant(""),  int  axis=-1,  bool  relax=false,  bool  tempclose=true,  bool  overwrite=false,  bool  reorder=false);
         bool fromarray(const  string& outfile=string(""),  const variant& pixels= initialize_variant(""),  const record& csys=initialize_record("") ,  bool  linear=false,  bool  overwrite=false,  bool  log=true);
         bool fromascii(const  string& outfile=string(""), const  string& infile=string(""),  const std::vector<int>& shape=initialize_vector(1, (int)-1), const  string& sep=string(":"),  const record& csys=initialize_record("") ,  bool  linear=false,  bool  overwrite=false);
         bool fromfits(const  string& outfile=string(""), const  string& infile=string(""),  int  whichrep=0,  int  whichhdu=0,  bool  zeroblanks=false,  bool  overwrite=false);
         bool fromimage(const  string& outfile=string(""), const  string& infile=string(""),  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""),  bool  dropdeg=false,  bool  overwrite=false);
         bool fromshape(const  string& outfile=string(""),  const std::vector<int>& shape=initialize_vector(1, (int)0),  const record& csys=initialize_record("") ,  bool  linear=false,  bool  overwrite=false,  bool  log=true, const  string& type=string("f"));
         bool maketestimage(const  string& outfile=string(""),  bool  overwrite=false);
         
                                      casac::image* adddegaxes(const  string& outfile=string(""),  bool  direction=false,  bool  spectral=false, const  string& stokes=string(""),  bool  linear=false,  bool  tabular=false,  bool  overwrite=false,  bool  silent=false);
         bool addnoise(const  string& type=string("normal"),  const std::vector<double>& pars=initialize_vector(2,(double)0.0, (double)1.0),  const variant& region= initialize_variant(""),  bool  zero=false,  const std::vector<int>& seeds=std::vector<int>() );
         
                                      casac::image* convolve(const  string& outfile=string(""),  const variant& kernel= initialize_variant(""), 
			  double scale=-1.0,  const variant& region = initialize_variant(""),  const variant& mask= initialize_variant(""),  bool  overwrite=false,  bool  stretch=false);
         record* boundingbox( const variant& region= initialize_variant(""));
         
                                      casac::image* boxcar(const  string& outfile=string(""),  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""),  int  axis=-1,  int  width=2,  bool  drop=true, const  string& dmethod=string("copy"),  bool  overwrite=false,  bool  stretch=false);
         string brightnessunit();
         bool calc(const  string& pixels,  bool  verbose=True);
         bool calcmask(const  string& mask=string(""), const  string& name=string(""),  bool  asdefault=true);
         bool close();
         
                                      casac::image* continuumsub(const  string& outline=string(""), const  string& outcont=string("continuumsub.im"),  const variant& region= initialize_variant(""),  const std::vector<int>& channels=initialize_vector(1, (int)-1), const  string& pol=string(""),  int  fitorder=0,  bool  overwrite=false);
         record* convertflux( const variant& value= initialize_variant("0Jy/beam"),  const variant& major= initialize_variant("1arcsec"),  const variant& minor= initialize_variant("1arcsec"), const  string& type=string("Gaussian"),  bool  topeak=true,  int  channel=-1,  int  polarization=-1);
         
                                      casac::image* convolve2d(const  string& outfile=string(""),  const std::vector<int>& axes=initialize_vector(2,(int)0, (int)1), const  string& type=string("gaussian"),  const variant& major= initialize_variant("0deg"),  const variant& minor= initialize_variant("0deg"),  const variant& pa= initialize_variant("0deg"), 
			  double scale=-1,  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""),  bool  overwrite=false,  bool  stretch=false,  bool  targetres=false,  const record& beam=initialize_record("") );
         
                                      casac::coordsys* coordsys( const std::vector<int>& axes=initialize_vector(1, (int)-1));
         record* coordmeasures( const std::vector<double>& pixel=initialize_vector(1, (double)-1), const  string& dframe=string("cl"), const  string& sframe=string("cl"));
         record* decompose( const variant& region = initialize_variant(""),  const variant& mask= initialize_variant(""),  bool  simple=false, 
			  double threshold=-1,  int  ncontour=11,  int  minrange=1,  int  naxis=2,  bool  fit=true, 
			  double maxrms=-1,  int  maxretry=-1,  int  maxiter=256, 
			  double convcriteria=0.0001,  bool  stretch=false);
         record* deconvolvecomponentlist( const record& complist,  int  channel=-1,  int  polarization=-1);
         record* deconvolvefrombeam( const variant& source= initialize_variant(""),  const variant& beam= initialize_variant(""));
         record* beamforconvolvedsize( const variant& source= initialize_variant(""),  const variant& convolved= initialize_variant(""));
         record* commonbeam();
         bool remove( bool  done=false,  bool  verbose=true);
         bool removefile(const  string& file);
         bool done( bool  remove=false,  bool  verbose=true);
         bool fft(const  string& real=string(""), const  string& imag=string(""), const  string& amp=string(""), const  string& phase=string(""),  const std::vector<int>& axes=initialize_vector(1, (int)-1),  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""),  bool  stretch=false, const  string& complex=string(""));
         record* findsources( int  nmax=20, 
			  double cutoff=0.1,  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""),  bool  point=true,  int  width=5,  bool  negfind=false);
         record* fitprofile(const  string& box=string(""),  const variant& region= initialize_variant(""), const  string& chans=string(""), const  string& stokes=string(""),  int  axis=-1,  const variant& mask= initialize_variant(""),  int  ngauss=1,  int  poly=-1, const  string& estimates=string(""),  int  minpts=1,  bool  multifit=false, const  string& model=string(""), const  string& residual=string(""), const  string& amp=string(""), const  string& amperr=string(""), const  string& center=string(""), const  string& centererr=string(""), const  string& fwhm=string(""), const  string& fwhmerr=string(""), const  string& integral=string(""), const  string& integralerr=string(""),  bool  stretch=false,  bool  logresults=true,  const variant& pampest= initialize_variant(""),  const variant& pcenterest= initialize_variant(""),  const variant& pfwhmest= initialize_variant(""),  const variant& pfix= initialize_variant(""),  const variant& gmncomps= initialize_variant("0"),  const variant& gmampcon= initialize_variant(""),  const variant& gmcentercon= initialize_variant(""),  const variant& gmfwhmcon= initialize_variant(""),  const std::vector<double>& gmampest=initialize_vector(1, (double)0.0),  const std::vector<double>& gmcenterest=initialize_vector(1, (double)0.0),  const std::vector<double>& gmfwhmest=initialize_vector(1, (double)0.0),  const variant& gmfix= initialize_variant(""),  const string& spxtype=string(""),  const std::vector<double>& spxest=std::vector<double>() ,  const std::vector<bool>&  spxfix=std::vector<bool>() ,  const variant& div= initialize_variant("0"), const  string& spxsol=string(""), const  string& spxerr=string(""), const  string& logfile=string(""),  bool  append=true,  const variant& pfunc= initialize_variant(""),  const std::vector<double>& goodamprange=initialize_vector(1, (double)0.0),  const std::vector<double>& goodcenterrange=initialize_vector(1, (double)0.0),  const std::vector<double>& goodfwhmrange=initialize_vector(1, (double)0.0),  const variant& sigma= initialize_variant(""), const  string& outsigma=string(""),  const std::vector<int>& planes=std::vector<int>() );
         record* fitcomponents(const  string& box=string(""),  const variant& region= initialize_variant(""),  const variant& chans= initialize_variant(""), const  string& stokes=string(""),  const variant& mask= initialize_variant(""),  const std::vector<double>& includepix=initialize_vector(1, (double)-1),  const std::vector<double>& excludepix=initialize_vector(1, (double)-1), const  string& residual=string(""), const  string& model=string(""), const  string& estimates=string(""), const  string& logfile=string(""),  bool  append=true, const  string& newestimates=string(""), const  string& complist=string(""),  bool  overwrite=false,  bool  dooff=false, 
			  double offset=0.0,  bool  fixoffset=false,  bool  stretch=false,  const variant& rms= initialize_variant(""),  const variant& noisefwhm= initialize_variant(""));
         bool fromrecord( const record& record, const  string& outfile=string(""));
         variant* getchunk( const std::vector<int>& blc=initialize_vector(1, (int)-1),  const std::vector<int>& trc=initialize_vector(1, (int)-1),  const std::vector<int>& inc=initialize_vector(1, (int)1),  const std::vector<int>& axes=initialize_vector(1, (int)-1),  bool  list=false,  bool  dropdeg=false,  bool  getmask=false);
         variant* getregion( const variant& region= initialize_variant(""),  const std::vector<int>& axes=initialize_vector(1, (int)-1),  const variant& mask = initialize_variant(""),  bool  list=false,  bool  dropdeg=false,  bool  getmask=false,  bool  stretch=false);
         record* getprofile( int  axis=-1, const  string& function=string("mean"),  const variant& region= initialize_variant(""), const  string& mask=string(""), const  string& unit=string(""),  bool  stretch=false, const  string& spectype=string("default"),  const variant& restfreq= initialize_variant(""), const  string& frame=string(""), const  string& logfile=string(""));
         record* getslice( const std::vector<double>& x,  const std::vector<double>& y,  const std::vector<int>& axes=initialize_vector(2,(int)0, (int)1),  const std::vector<int>& coord=initialize_vector(1, (int)-1),  int  npts=0, const  string& method=string("linear"));
         
                                      casac::image* hanning(const  string& outfile=string(""),  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""),  int  axis=-10,  bool  drop=true,  bool  overwrite=false,  bool  async=false,  bool  stretch=false, const  string& dmethod=string("copy"));
         std::vector<bool> haslock();
         record* histograms( const std::vector<int>& axes=initialize_vector(1, (int)-1),  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""),  int  nbins=25,  const std::vector<double>& includepix=initialize_vector(1, (double)-1),  bool  cumu=false,  bool  log=false,  bool  stretch=false);
         std::vector<std::string> history( bool  list=true);
         bool insert(const  string& infile=string(""),  const variant& region= initialize_variant(""),  const std::vector<double>& locate=initialize_vector(1, (double)-1),  bool  verbose=false);
         bool isopen();
         bool ispersistent();
         bool lock( bool  writelock=false,  int  nattempts=0);
         bool makecomplex(const  string& outfile, const  string& imag,  const variant& region= initialize_variant(""),  bool  overwrite=false);
         std::vector<std::string> maskhandler(const  string& op=string("default"),  const std::vector<std::string>& name=std::vector<string>(1, ""));
         record* miscinfo();
         bool modify( const record& model,  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""),  bool  subtract=true,  bool  list=true,  bool  stretch=false);
         record* maxfit( const variant& region= initialize_variant(""),  bool  point=true,  int  width=5,  bool  negfind=false,  bool  list=true);
         
                                      casac::image* moments( const std::vector<int>& moments=initialize_vector(1, (int)0),  int  axis=-10,  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""),  const std::vector<std::string>& method=std::vector<string>(1, ""),  const std::vector<int>& smoothaxes=initialize_vector(1, (int)-1),  const variant& smoothtypes= initialize_variant(""),  const std::vector<double>& smoothwidths=initialize_vector(1, (double)0.0),  const std::vector<double>& includepix=initialize_vector(1, (double)-1),  const std::vector<double>& excludepix=initialize_vector(1, (double)-1), 
			  double peaksnr=3.0, 
			  double stddev=0.0, const  string& doppler=string("RADIO"), const  string& outfile=string(""), const  string& smoothout=string(""),  bool  overwrite=false,  bool  drop=true,  bool  stretch=false,  bool  async=false);
         string name( bool  strippath=false);
         bool open(const  string& infile);
         
                                      casac::image* pad(const  string& outfile=string(""),  int  npixels=1, 
			  double value=0,  bool  padmask=false,  bool  overwrite=false,  const variant& region= initialize_variant(""), const  string& box=string(""), const  string& chans=string(""), const  string& stokes=string(""), const  string& mask=string(""),  bool  stretch=false,  bool  wantreturn=true);
         
                                      casac::image* crop(const  string& outfile=string(""),  const std::vector<int>& axes=std::vector<int>() ,  bool  overwrite=false,  const variant& region= initialize_variant(""), const  string& box=string(""), const  string& chans=string(""), const  string& stokes=string(""), const  string& mask=string(""),  bool  stretch=false,  bool  wantreturn=true);
         record* pixelvalue( const std::vector<int>& pixel=initialize_vector(1, (int)-1));
         bool putchunk( const variant& pixels,  const std::vector<int>& blc=initialize_vector(1, (int)-1),  const std::vector<int>& inc=initialize_vector(1, (int)1),  bool  list=false,  bool  locking=true,  bool  replicate=false);
         bool putregion( const variant& pixels= initialize_variant(""),  const variant& pixelmask= initialize_variant(""),  const variant& region= initialize_variant(""),  bool  list=false,  bool  usemask=true,  bool  locking=true,  bool  replicate=false);
         
                                      casac::image* rebin(const  string& outfile,  const std::vector<int>& bin,  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""),  bool  dropdeg=false,  bool  overwrite=false,  bool  async=false,  bool  stretch=false,  bool  crop=false);
         
                                      casac::image* regrid(const  string& outfile=string(""),  const std::vector<int>& shape=initialize_vector(1, (int)-1),  const record& csys=initialize_record("") ,  const std::vector<int>& axes=initialize_vector(1, (int)-1),  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""), const  string& method=string("linear"),  int  decimate=10,  bool  replicate=false,  bool  doref=true,  bool  dropdeg=false,  bool  overwrite=false,  bool  force=false,  bool  asvelocity=false,  bool  async=false,  bool  stretch=false);
         
                                      casac::image* transpose(const  string& outfile=string(""),  const variant& order= initialize_variant(""));
         
                                      casac::image* rotate(const  string& outfile=string(""),  const std::vector<int>& shape=initialize_vector(1, (int)-1),  const variant& pa= initialize_variant("0deg"),  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""), const  string& method=string("cubic"),  int  decimate=0,  bool  replicate=false,  bool  dropdeg=false,  bool  overwrite=false,  bool  stretch=false);
         bool rotatebeam( const variant& angle= initialize_variant("0deg"));
         bool rename(const  string& name,  bool  overwrite=false);
         bool replacemaskedpixels( const variant& pixels,  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""),  bool  update=false,  bool  list=false,  bool  stretch=false);
         record* restoringbeam( int  channel=-1,  int  polarization=-1);
         
                                      casac::image* sepconvolve(const  string& outfile=string(""),  const std::vector<int>& axes=initialize_vector(1, (int)-1),  const std::vector<std::string>& types=std::vector<string>(1, ""),  const variant& widths= initialize_variant(""), 
			  double scale=-1,  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""),  bool  overwrite=false,  bool  stretch=false);
         bool set( const variant& pixels = initialize_variant(""),  int  pixelmask=-1,  const variant& region= initialize_variant(""),  bool  list=false);
         bool setbrightnessunit(const  string& unit);
         bool setcoordsys( const record& csys);
         bool sethistory(const  string& origin=string(""),  const std::vector<std::string>& history=std::vector<string>(1, ""));
         bool setmiscinfo( const record& info);
         std::vector<int> shape();
         bool setrestoringbeam( const variant& major = initialize_variant("1arcsec"),  const variant& minor = initialize_variant("1arcsec"),  const variant& pa = initialize_variant("0deg"),  const record& beam=initialize_record("") ,  bool  remove=false,  bool  log=true,  int  channel=-1,  int  polarization=-1, const  string& imagename=string(""));
         record* statistics( const std::vector<int>& axes=initialize_vector(1, (int)-1),  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""),  const std::vector<double>& includepix=initialize_vector(1, (double)-1),  const std::vector<double>& excludepix=initialize_vector(1, (double)-1),  bool  list=false,  bool  force=false,  bool  disk=false,  bool  robust=false,  bool  verbose=false,  bool  stretch=false, const  string& logfile=string(""),  bool  append=true, const  string& algorithm=string("classic"), 
			  double fence=-1, const  string& center=string("mean"),  bool  lside=true, 
			  double zscore=-1,  int  maxiter=-1, const  string& clmethod=string("auto"));
         bool twopointcorrelation(const  string& outfile=string(""),  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""),  const std::vector<int>& axes=initialize_vector(1, (int)-1), const  string& method=string("structurefunction"),  bool  overwrite=false,  bool  stretch=false);
         
                                      casac::image* subimage(const  string& outfile=string(""),  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""),  bool  dropdeg=false,  bool  overwrite=false,  bool  list=true,  bool  stretch=false,  bool  wantreturn=true,  const std::vector<int>& keepaxes=std::vector<int>() );
         record* summary(const  string& doppler=string("RADIO"),  bool  list=true,  bool  pixelorder=true,  bool  verbose=false);
         bool tofits(const  string& outfile=string(""),  bool  velocity=false,  bool  optical=true,  int  bitpix=-32, 
			  double minpix=1, 
			  double maxpix=-1,  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""),  bool  overwrite=false,  bool  dropdeg=false,  bool  deglast=false,  bool  dropstokes=false,  bool  stokeslast=true,  bool  wavelength=false,  bool  airwavelength=false,  bool  async=false,  bool  stretch=false,  bool  history=true);
         bool toASCII(const  string& outfile,  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""), const  string& sep=string(":"), const  string& format=string("\%e"), 
			  double maskvalue=-999,  bool  overwrite=false,  bool  stretch=false);
         record* torecord();
         string type();
         record* topixel( const variant& value);
         record* toworld( const variant& value= initialize_variant(""), const  string& format=string("n"),  bool  dovelocity=True);
         bool unlock();
         
                                      casac::image* newimagefromarray(const  string& outfile=string(""),  const variant& pixels= initialize_variant(""),  const record& csys=initialize_record("") ,  bool  linear=false,  bool  overwrite=false,  bool  log=true);
         
                                      casac::image* newimagefromfits(const  string& outfile=string(""), const  string& infile=string(""),  int  whichrep=0,  int  whichhdu=0,  bool  zeroblanks=false,  bool  overwrite=false);
         
                                      casac::image* newimagefromimage(const  string& infile=string(""), const  string& outfile=string(""),  const variant& region= initialize_variant(""),  const variant& mask= initialize_variant(""),  bool  dropdeg=false,  bool  overwrite=false);
         
                                      casac::image* newimagefromshape(const  string& outfile=string(""),  const std::vector<int>& shape=initialize_vector(1, (int)0),  const record& csys=initialize_record("") ,  bool  linear=false,  bool  overwrite=false,  bool  log=true, const  string& type=string("f"));
         
                                      casac::image* pbcor( const variant& pbimage= initialize_variant(""), const  string& outfile=string(""),  bool  overwrite=false, const  string& box=string(""),  const variant& region= initialize_variant(""), const  string& chans=string(""), const  string& stokes=string(""), const  string& mask=string(""), const  string& mode=string("divide"),  float  cutoff=-1.0,  bool  stretch=false);
         
                                      casac::image* pv(const  string& outfile=string(""),  const variant& start= initialize_variant(""),  const variant& end= initialize_variant(""),  const variant& center= initialize_variant(""),  const variant& length= initialize_variant(""),  const variant& pa= initialize_variant(""),  const variant& width= initialize_variant("1"), const  string& unit=string("arcsec"),  bool  overwrite=false,  const variant& region= initialize_variant(""), const  string& chans=string(""), const  string& stokes=string(""), const  string& mask=string(""),  bool  stretch=false,  bool  wantreturn=true);
         variant* makearray(
			  double v=0.0,  const std::vector<int>& shape=initialize_vector(1, (int)0));
         bool isconform(const  string& other);

      private :
#include <tools/images/image_private.h>
   }; 

 
}
#endif
