  
#ifndef _simulator_cmpt_h_
#define _simulator_cmpt_h_

#include <vector>
#include <string>
#include <complex>
#include <stdcasa/record.h>
#include <tools/casaswig_types.h>
#include <casa/Exceptions/Error.h>
#include <tools/synthesis/simulator_forward.h>

using namespace std;

namespace casac {
           


   class  simulator  {
       public :
simulator();
         ~simulator();
         bool open(const  string& ms=string(""));
         bool openfromms(const  string& ms=string(""));
         bool close();
         bool done();
         string name();
         bool summary();
         string type();
         bool settimes( const variant& integrationtime= initialize_variant("10s"),  bool  usehourangle=true,  const variant& referencetime= initialize_variant("50000.0d"));
         bool observe(const  string& sourcename=string(""), const  string& spwname=string(""),  const variant& starttime= initialize_variant("0s"),  const variant& stoptime= initialize_variant("3600s"),  bool  add_observation=false,  bool  state_sig=true,  bool  state_ref=false, 
			  double state_cal=0.0, 
			  double state_load=0.0,  int  state_sub_scan=0, const  string& state_obs_mode=string("OBSERVE_TARGET.ON_SOURCE"), const  string& observer=string("CASA simulator"), const  string& project=string("CASA simulation"));
         bool observemany( const std::vector<std::string>& sourcenames=std::vector<string>(1, ""), const  string& spwname=string(""),  const std::vector<std::string>& starttimes=std::vector<string>(1, ""),  const std::vector<std::string>& stoptimes=std::vector<string>(1, ""),  const std::vector<std::string>& directions=std::vector<string>(1, ""),  bool  add_observation=false,  bool  state_sig=true,  bool  state_ref=false, 
			  double state_cal=0.0, 
			  double state_load=0.0,  int  state_sub_scan=0, const  string& state_obs_mode=string("OBSERVE_TARGET#ON_SOURCE"), const  string& observer=string("CASA simulator"), const  string& project=string("CASA simulation"));
         bool setlimits(
			  double shadowlimit=1e-6,  const variant& elevationlimit= initialize_variant("10deg"));
         bool setauto(
			  double autocorrwt=0.0);
         bool setconfig(const  string& telescopename=string("VLA"),  const std::vector<double>& x=initialize_vector(1, (double)0),  const std::vector<double>& y=initialize_vector(1, (double)0),  const std::vector<double>& z=initialize_vector(1, (double)0),  const std::vector<double>& dishdiameter=initialize_vector(1, (double)0),  const std::vector<double>& offset=initialize_vector(1, (double)0),  const std::vector<std::string>& mount=std::vector<string>(1, ""),  const std::vector<std::string>& antname=std::vector<string>(1, ""),  const std::vector<std::string>& padname=std::vector<string>(1, ""), const  string& coordsystem=string("global"),  const variant& referencelocation= initialize_variant("ALMA"));
         bool setknownconfig(const  string& arrayname=string("VLA"));
         bool setfeed(const  string& mode=string(""),  const std::vector<double>& x=initialize_vector(1, (double)0),  const std::vector<double>& y=initialize_vector(1, (double)0),  const std::vector<std::string>& pol=std::vector<string>(1, ""));
         bool setfield(const  string& sourcename=string("SOURCE"),  const variant& sourcedirection= initialize_variant(""), const  string& calcode=string(""),  const variant& distance= initialize_variant("0m"));
         bool setmosaicfield(const  string& sourcename=string("SOURCE"), const  string& calcode=string(""),  const variant& fieldcenter= initialize_variant(""),  int  xmosp=1,  int  ymosp=1,  const variant& mosspacing= initialize_variant("1arcsec"),  const variant& distance= initialize_variant("0m"));
         bool setspwindow(const  string& spwname=string("XBAND"),  const variant& freq= initialize_variant("8.0e9Hz"),  const variant& deltafreq= initialize_variant("50e6Hz"),  const variant& freqresolution= initialize_variant("50.e6Hz"), const  string& refcode=string("TOPO"),  int  nchannels=1, const  string& stokes=string("RR LL"));
         bool setdata( const std::vector<int>& spwid=initialize_vector(1, (int)0),  const std::vector<int>& fieldid=initialize_vector(1, (int)0), const  string& msselect=string(""));
         bool predict( const std::vector<std::string>& imagename=std::vector<string>(1, ""), const  string& complist=string(""),  bool  incremental=false);
         bool setoptions(const  string& ftmachine=string("ft"),  int  cache=0,  int  tile=16, const  string& gridfunction=string("SF"),  const variant& location= initialize_variant("ALMA"), 
			  double padding=1.3,  int  facets=1, 
			  double maxdata=2000.0,  int  wprojplanes=1);
         bool setvp( bool  dovp=true,  bool  usedefaultvp=true, const  string& vptable=string(""),  bool  dosquint=true,  const variant& parangleinc= initialize_variant("360deg"),  const variant& skyposthreshold= initialize_variant("180deg"), 
			  double pblimit=1.0e-2);
         bool corrupt();
         bool reset();
         bool setbandpass(const  string& mode=string("calculate"), const  string& table=string(""),  const variant& interval= initialize_variant("3600s"),  const std::vector<double>& amplitude=initialize_vector(1, (double)0.0));
         bool setapply( const string& table=string(""),  const string& type=string(""), 
			  double t=0.0,  const variant& field = initialize_variant(""),  const string& interp=string("linear"),  bool  calwt=false,  const  std::vector<int>& spwmap=initialize_vector(1, (int)-1), 
			  double opacity=0.0);
         bool setgain(const  string& mode=string("fbm"), const  string& table=string(""),  const variant& interval= initialize_variant("10s"),  const std::vector<double>& amplitude=initialize_vector(1, (double)0.01));
         bool settrop(const  string& mode=string("screen"), const  string& table=string(""), 
			  double pwv=3.0, 
			  double deltapwv=0.15, 
			  double beta=1.1, 
			  double windspeed=7.);
         bool setpointingerror(const  string& epjtablename=string(""),  bool  applypointingoffsets=false,  bool  dopbcorrection=false);
         bool setleakage(const  string& mode=string("constant"), const  string& table=string(""),  const std::vector<double>& amplitude=initialize_vector(1, (double)0.01),  const std::vector<double>& offset=initialize_vector(1, (double)0.));
         bool oldsetnoise(const  string& mode=string("calculate"), const  string& table=string(""),  const variant& simplenoise= initialize_variant("0.0Jy"), 
			  double antefficiency=0.8, 
			  double correfficiency=0.85, 
			  double spillefficiency=0.85, 
			  double tau=0.1, 
			  double trx=50, 
			  double tatmos=230.0, 
			  double tcmb=2.7);
         bool setnoise(const  string& mode=string("simplenoise"), const  string& table=string(""),  const variant& simplenoise= initialize_variant("0.1Jy"),  const variant& pground= initialize_variant("560mbar"), 
			  double relhum=20.0,  const variant& altitude= initialize_variant("5000m"),  const variant& waterheight= initialize_variant("200m"),  const variant& pwv= initialize_variant("1mm"), 
			  double tatmos=250.0, 
			  double tau=0.1, 
			  double antefficiency=0.8, 
			  double spillefficiency=0.85, 
			  double correfficiency=0.88, 
			  double trx=50, 
			  double tground=270.0, 
			  double tcmb=2.73,  bool  OTF=true, 
			  double senscoeff=0.,  int  rxtype=0);
         bool setpa(const  string& mode=string("calculate"), const  string& table=string(""),  const variant& interval= initialize_variant("10s"));
         bool setseed( int  seed=185349251);

      private :
#include <tools/synthesis/simulator_private.h>
   }; 

 
}
#endif
