  
#ifndef _vpmanager_cmpt_h_
#define _vpmanager_cmpt_h_

#include <vector>
#include <string>
#include <complex>
#include <stdcasa/record.h>
#include <tools/casaswig_types.h>
#include <casa/Exceptions/Error.h>
#include <tools/synthesis/vpmanager_forward.h>

using namespace std;

namespace casac {
           


   class  vpmanager  {
       public :
vpmanager();
         ~vpmanager();
         bool saveastable(const  string& tablename=string(""));
         bool loadfromtable(const  string& tablename=string(""));
         bool summarizevps( bool  verbose=false);
         record* setcannedpb(const  string& telescope=string("VLA"), const  string& othertelescope=string(""),  bool  dopb=true, const  string& commonpb=string("DEFAULT"),  bool  dosquint=false,  const variant& paincrement= initialize_variant("720deg"),  bool  usesymmetricbeam=false);
         record* setpbairy(const  string& telescope=string("VLA"), const  string& othertelescope=string(""),  bool  dopb=true,  const variant& dishdiam= initialize_variant("25.0m"),  const variant& blockagediam= initialize_variant("2.5m"),  const variant& maxrad= initialize_variant("0.8deg"),  const variant& reffreq= initialize_variant("1.0GHz"),  const variant& squintdir= initialize_variant(""),  const variant& squintreffreq= initialize_variant("1.0GHz"),  bool  dosquint=false,  const variant& paincrement= initialize_variant("720deg"),  bool  usesymmetricbeam=false);
         record* setpbcospoly(const  string& telescope=string("VLA"), const  string& othertelescope=string(""),  bool  dopb=true,  const std::vector<double>& coeff=initialize_vector(1, (double)-1),  const std::vector<double>& scale=initialize_vector(1, (double)-1),  const variant& maxrad= initialize_variant("0.8deg"),  const variant& reffreq= initialize_variant("1.0GHz"), const  string& isthispb=string("PB"),  const variant& squintdir= initialize_variant(""),  const variant& squintreffreq= initialize_variant("1.0GHz"),  bool  dosquint=false,  const variant& paincrement= initialize_variant("720deg"),  bool  usesymmetricbeam=false);
         record* setpbgauss(const  string& telescope=string("VLA"), const  string& othertelescope=string(""),  bool  dopb=true,  const variant& halfwidth= initialize_variant("0.5deg"),  const variant& maxrad= initialize_variant("1.0deg"),  const variant& reffreq= initialize_variant("1.0GHz"), const  string& isthispb=string("PB"),  const variant& squintdir= initialize_variant(""),  const variant& squintreffreq= initialize_variant("1.0GHz"),  bool  dosquint=false,  const variant& paincrement= initialize_variant("720deg"),  bool  usesymmetricbeam=false);
         record* setpbinvpoly(const  string& telescope=string("VLA"), const  string& othertelescope=string(""),  bool  dopb=true,  const std::vector<double>& coeff=initialize_vector(1, (double)-1),  const variant& maxrad= initialize_variant("0.8deg"),  const variant& reffreq= initialize_variant("1.0GHz"), const  string& isthispb=string("PB"),  const variant& squintdir= initialize_variant(""),  const variant& squintreffreq= initialize_variant("1.0"),  bool  dosquint=false,  const variant& paincrement= initialize_variant("720deg"),  bool  usesymmetricbeam=false);
         record* setpbnumeric(const  string& telescope=string("VLA"), const  string& othertelescope=string(""),  bool  dopb=true,  const std::vector<double>& vect=initialize_vector(1, (double)-1),  const variant& maxrad= initialize_variant("0.8deg"),  const variant& reffreq= initialize_variant("1.0GHz"), const  string& isthispb=string("PB"),  const variant& squintdir= initialize_variant(""),  const variant& squintreffreq= initialize_variant("1.0GHz"),  bool  dosquint=false,  const variant& paincrement= initialize_variant("720deg"),  bool  usesymmetricbeam=false);
         record* setpbimage(const  string& telescope=string("VLA"), const  string& othertelescope=string(""),  bool  dopb=true, const  string& realimage=string(""), const  string& imagimage=string(""), const  string& compleximage=string(""),  const std::vector<std::string>& antnames=std::vector<string>(1, ""));
         record* setpbpoly(const  string& telescope=string("VLA"), const  string& othertelescope=string(""),  bool  dopb=true,  const std::vector<double>& coeff=initialize_vector(1, (double)-1),  const variant& maxrad= initialize_variant("0.8deg"),  const variant& reffreq= initialize_variant("1.0GHz"), const  string& isthispb=string("PB"),  const variant& squintdir= initialize_variant(""),  const variant& squintreffreq= initialize_variant("1.0GHz"),  bool  dosquint=false,  const variant& paincrement= initialize_variant("720"),  bool  usesymmetricbeam=false);
         bool setpbantresptable(const  string& telescope=string(""), const  string& othertelescope=string(""),  bool  dopb=true, const  string& antresppath=string(""));
         bool reset();
         bool setuserdefault( int  vplistnum=-1, const  string& telescope=string(""), const  string& anttype=string(""));
         int getuserdefault(const  string& telescope=string(""), const  string& anttype=string(""));
         std::vector<std::string> getanttypes(const  string& telescope=string(""),  const variant& obstime = initialize_variant(""),  const variant& freq = initialize_variant(""),  const variant& obsdirection = initialize_variant("AZEL 0deg 90deg"));
         int numvps(const  string& telescope=string(""),  const variant& obstime = initialize_variant(""),  const variant& freq = initialize_variant(""),  const variant& obsdirection = initialize_variant("AZEL 0deg 90deg"));
         record* getvp(const  string& telescope=string(""), const  string& antennatype=string(""),  const variant& obstime = initialize_variant(""),  const variant& freq = initialize_variant(""),  const variant& obsdirection = initialize_variant("AZEL 0deg 90deg"));
         bool createantresp( const string& imdir=string(""),  const string& starttime=string(""),  const std::vector<std::string>& bandnames=std::vector<string>(1, ""),  const std::vector<std::string>& bandminfreq=std::vector<string>(1, ""),  const std::vector<std::string>& bandmaxfreq=std::vector<string>(1, ""));
         string getrespimagename( const string& telescope=string(""),  const string& starttime=string(""), const  string& frequency=string(""), const  string& functype=string("ANY"), const  string& anttype=string(""), const  string& azimuth=string("0deg"), const  string& elevation=string("45deg"), const  string& rectype=string(""),  int  beamnumber=0);

      private :
#include <tools/synthesis/vpmanager_private.h>
   }; 

 
}
#endif
