// -*- mode: c++ -*-
#ifndef VI2_CHUNK_STATISTICS_ITERATEE_H_
#define VI2_CHUNK_STATISTICS_ITERATEE_H_

#include <msvis/MSVis/VisBuffer2.h>
#include <casacore/scimath/Mathematics/StatisticsAlgorithm.h>

namespace casa {

// This class is used to encapsulate the action(s) taken on a sequence of
// StatisticsAlgorithm instances that are generated by the
// Vi2ChunkDataProvider::foreachChunk() method, which can be used to compute
// statistics for each MS chunk provided by a VisibilityIterator2 instance. See
// the Vi2ChunkDataProvider.h file for an outline of how this class may be used.

template <class DataIterator, class WeightsIterator, class MaskIterator>
class Vi2ChunkStatisticsIteratee {

	typedef typename DataIterator::AccumType AccumType;

public:

	// This method will be called by Vi2ChunkDataProvider::foreachChunk() for
	// each chunk provided by a VisibilityIterator2 instance, with the
	// StatisticsAlgorithm instance initialized with a data provider for the
	// chunk. The pointer to the VisBuffer2 instance is provided in case this
	// method needs to get some metadata for the current chunk.
	virtual void nextChunk(
		StatisticsAlgorithm<AccumType,DataIterator,MaskIterator,WeightsIterator> &stats,
		const vi::VisBuffer2 *vb) = 0;
};

} // namespace casa

#endif // VI2_CHUNK_STATISTICS_ITERATEE_H_
